import * as React from "react";

import { Link } from "react-router-dom";

import Grid from "@material-ui/core/Grid";
import Typography from "@material-ui/core/Typography";
import Button from "@material-ui/core/Button";
import Paper from "@material-ui/core/Paper";
import CircularProgress from "@material-ui/core/CircularProgress";

import Scoreboard from "../components/scoreboard";
import SummaryTable from "../components/SummaryTable";

import { ILevel } from "../models/level";
import { ILearner, TopTen } from "../models/learner";
import { IReviewMetadata } from "../models/review";

interface IProps {
    getNextLevel: () => Promise<ILevel>;
    getLastReview: () => Promise<IReviewMetadata>;
    getTopTen: () => Promise<ILearner[]>;

    nextLevel: ILevel;
    loadingNextLevel: boolean;
    setLoadingNL: (state: boolean) => void;
    setNextLevel: (level: ILevel) => void;
    topTen: TopTen[];
    loadingTopTen: boolean;
    setLoadingTT: (state: boolean) => void;
    setTopTen: (topten: TopTen[]) => void;
    lastReview: IReviewMetadata;
    loadingLastReview: boolean;
    setLoadingLR: (state: boolean) => void;
    setLastReview: (review: IReviewMetadata) => void;
}

export default class Dashboard extends React.Component<IProps> {
    componentDidMount() {
        this.props.setLoadingNL(true);

        this.props.getNextLevel().then(res => {
            this.props.setLoadingNL(false);
            this.props.setNextLevel(res);
        }, err => {
            console.log("Failed to fetch next level!", err);
        });

        this.props.getTopTen().then(res => {
            this.props.setLoadingTT(false);
            this.props.setTopTen(res);
        }, err => {
            console.log("Failed to fetch Top Ten");
        });

        this.props.getLastReview().then(res => {
            this.props.setLoadingLR(false);
            this.props.setLastReview(res);
        }, err => {
            console.log("Failed to fetch Last Review");
        });
    }

    render() {
        const small = window.matchMedia("(max-width: 700px)").matches;
        const direction = small ? "column" : "row";

        const level = this.props.nextLevel;

        return <div>
            <Grid container direction={direction} spacing={16}>
                <Grid item lg={4}>
                    <Paper className="paper">
                        {this.props.loadingNextLevel ? (
                            <CircularProgress />
                        ) : (
                                <div>
                                    <Typography variant="title">{`Level ${level.level}`}</Typography>
                                    <Typography variant="title" component="p">{level.name}</Typography>
                                    <br />
                                    <Typography component="p">
                                        {level.desc}
                                    </Typography>
                                    <Button
                                        component={Link}
                                        to={`/level/${level.level}`}
                                        className="lesson-card-btn">
                                        Zum Level
                                    </Button>
                                </div>

                            )}
                    </Paper>
                </Grid>
                <Grid item lg={4}>
                    <Paper className="paper">
                        {this.props.loadingTopTen ? (
                            <CircularProgress />
                        ) : (
                                <div>
                                    <Typography variant="title" component="p">
                                        Rangliste: Top 10
                                  </Typography>

                                    <Scoreboard topTen={this.props.topTen} />
                                </div>
                            )}
                    </Paper>
                </Grid>
                <Grid item lg={4}>
                    <Paper className="paper">
                        {
                            this.props.loadingLastReview ? (
                                <CircularProgress />
                            ) : (
                                    <div>
                                        <Typography variant="title">
                                            Letzte Wiederholung
                                        </Typography>
                                        <SummaryTable reviewMeta={this.props.lastReview} />

                                        <Button
                                            component={Link}
                                            to="/review/queue"
                                            className="lesson-card-btn">
                                            Vokabeln üben
                                        </Button>
                                    </div>
                                )
                        }
                    </Paper>
                </Grid>
            </Grid>
        </div>;
    }
};
